package com.example.fieldagent.ui.homescreen

import android.annotation.SuppressLint
import android.content.Intent
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.PopupMenu
import android.widget.PopupWindow
import android.widget.TextView
import androidx.activity.viewModels
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.example.fieldagent.R
import com.example.fieldagent.databinding.FragmentHomeBinding
import com.example.fieldagent.ui.homescreen.adapter.HomeAdapter
import androidx.core.graphics.drawable.toDrawable
import androidx.fragment.app.viewModels
import com.consultantapp.data.network.responseUtil.Status
import com.example.fieldagent.data.network.ApisRespHandler
import com.example.fieldagent.ui.login.LoginActivity
import com.example.fieldagent.ui.login.LoginViewModel
import com.example.fieldagent.utils.IS_LOGIN
import com.example.fieldagent.utils.PrefsManager
import com.example.fieldagent.utils.USER_DATA
import com.example.fieldagent.utils.dialogs.ProgressDialog
import dagger.hilt.android.AndroidEntryPoint
import java.util.HashMap
import javax.inject.Inject
import androidx.lifecycle.Observer
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.RecyclerView
import com.example.fieldagent.data.models.responses.UserData
import com.example.fieldagent.databinding.PopLayoutBinding
import com.example.fieldagent.utils.AFTER
import com.example.fieldagent.utils.PER_PAGE
import com.example.fieldagent.utils.PER_PAGE_LOAD
import com.example.fieldagent.utils.gone
import com.example.fieldagent.utils.hideShowView
import com.example.fieldagent.utils.isConnectedToInternet
import com.example.fieldagent.utils.visible
import kotlinx.coroutines.Job
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch

@AndroidEntryPoint
class HomeFragment : Fragment() {

    private var _binding: FragmentHomeBinding? = null

    private val binding get() = _binding!!
    private lateinit var adapter: HomeAdapter

    @Inject
    lateinit var prefsManager: PrefsManager

    private  val viewModel: HomeViewModel by viewModels()
    private lateinit var progressDialog: ProgressDialog

    private var items = ArrayList<UserData>()

    private var isLastPage = false

    private var isFirstPage = true

    private var isLoadingMoreItems = false

    private var searchJob: Job? = null


    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {

        _binding = FragmentHomeBinding.inflate(inflater, container, false)
        return binding.root

    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        initialize()
        bindObservers()

        setAdapter()
        listener()

    }


    private  fun initialize(){
        prefsManager.save(IS_LOGIN,true)

        val hashMap = HashMap<String, String>()
        hashMap["limit"] = "10"
        hashMap["Page"] = "1"
        viewModel.getInspectionList(hashMap);

    }
    private fun setAdapter(){
        binding.rvList.layoutManager = LinearLayoutManager(requireActivity())
        adapter = HomeAdapter(this,items)
        binding.rvList.adapter = adapter
        binding.rvList.itemAnimator = null
    }

    private  fun listener(){


        binding.llBtn.setOnClickListener {
            findNavController().navigate(R.id.addSiteFragment)
        }
        binding.llLogoutBtn.setOnClickListener {

            val intent = Intent(requireActivity(), LoginActivity::class.java)
            startActivity(intent)
            requireActivity().finish()
        }

        binding.ivFilter.setOnClickListener {


            val inflater = LayoutInflater.from(binding.ivFilter.context)
            val parent = binding.root as ViewGroup
            val popupView = PopLayoutBinding.inflate(inflater,parent,false)

            val popupWindow = PopupWindow(
                popupView.root,
                ViewGroup.LayoutParams.WRAP_CONTENT,
                ViewGroup.LayoutParams.WRAP_CONTENT,
                true // focusable
            )


            popupView.txtCompleted.setOnClickListener {
                refreshPagination()
                hitApi(false)
            }

            popupView.txtInProgress.setOnClickListener {
                refreshPagination()
                hitApi(false)
            }


            popupWindow.setBackgroundDrawable(Color.TRANSPARENT.toDrawable())
            popupWindow.isOutsideTouchable = true
            popupWindow.elevation = 40f

            popupWindow.showAsDropDown(binding.ivFilter)
        }


        binding.swipeRefresh.setOnRefreshListener {
            hitApi(true)
        }



        binding.rvList.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)

                val layoutManager = binding.rvList.layoutManager as LinearLayoutManager
                val totalItemCount = layoutManager.itemCount - 1
                val lastVisibleItemPosition = layoutManager.findLastVisibleItemPosition()

                if (!isLoadingMoreItems && !isLastPage && lastVisibleItemPosition >= totalItemCount) {
                    isLoadingMoreItems = true
                    hitApi(false)
                }
            }
        })

        binding.etSearch.addTextChangedListener(object : TextWatcher {
            override fun afterTextChanged(s: Editable?) {
                // Debounce: cancel any previous search job
                searchJob?.cancel()
                searchJob = lifecycleScope.launch {
                    delay(300) // wait for user to stop typing (300ms)
                    val query = s.toString()
                    if (query.isNotBlank()) {
                        refreshPagination()
                       hitApi(false)
                    }
                }
            }

            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {}
        })

    }

    private fun refreshPagination(){
        isLastPage = false
        isFirstPage = true
        isLoadingMoreItems = false
    }

    private fun hitApi(firstHit: Boolean) {
        if (isConnectedToInternet(requireContext(), true)) {
            if (firstHit) {
                isFirstPage = true
                isLastPage = false
            }

            val hashMap = HashMap<String, String>()
            if (!isFirstPage && items.isNotEmpty())
               // hashMap[AFTER] = items[items.size - 1].id ?: ""

            hashMap[PER_PAGE] = PER_PAGE_LOAD.toString()

            viewModel.getInspectionList(hashMap)
        } else
            binding.swipeRefresh.isRefreshing = false
    }
    private fun bindObservers() {
        viewModel.inspections.observe(requireActivity(), Observer {
            it ?: return@Observer
            when (it.status) {
                Status.SUCCESS -> {
                    binding.clLoader.root.gone()
                    binding.swipeRefresh.isRefreshing = false

                    isLoadingMoreItems = false

                   // val tempList = it.data?.notifications ?: emptyList()
                    val tempList=ArrayList<UserData>()
                    if (isFirstPage) {
                        isFirstPage = false
                        items.clear()
                        items.addAll(tempList)

                        adapter.notifyDataSetChanged()
                    } else {
                        val oldSize = items.size
                        items.addAll(tempList)

                        adapter.notifyItemRangeInserted(oldSize, items.size)
                    }

                    isLastPage = tempList.size < PER_PAGE_LOAD
                    adapter.setAllItemsLoaded(isLastPage)


                    binding.clNoData.rootView.hideShowView(items.isEmpty())
                }
                Status.ERROR -> {
                    isLoadingMoreItems = false
                    adapter.setAllItemsLoaded(true)

                    binding.swipeRefresh.isRefreshing = false
                    binding.clLoader.root.gone()
                    ApisRespHandler.handleError(it.error, requireActivity())
                }
                Status.LOADING -> {
                    if (!isLoadingMoreItems && !binding.swipeRefresh.isRefreshing)
                        binding.clLoader.root.visible()
                }
            }
        })
    }


    fun clickItem(pos: Int) {
        val item = items[pos]

    }


    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

}